//t3dApiDemo.cpp  - contains a function to demonstrate the use of 3D Meshing API

#include <stdio.h>
#include <stdlib.h>
#include <memory.h>
#include "t3dMeshIo.h"
#include "t3dTetMain.h"

/*  A function to demonstrate the use of the 3D Meshing API.

    The input mesh geometry is the same geometry as stored in 'cube.g3d'
    It contains 8 points and 12 triangles.
    Note the orientation of the triangles is such that the normal points outwards.

    This function makes use of the API,
    
    int t3dCreateTetMesh (const t3dMeshIo* pMeshIn, t3dMeshIo* pMeshOut, eTetMeshResult* peResult)

*/

int t3dApiDemo()
{
  int iRet=0;
  //Initialize eResult to eFailed.
  eTetMeshResult eResult = eFailed;

  //Local variables storing the 3D Surface Mesh
  int iNoPts = 8;
  int iNoTri = 12;

  const double adCoord [] = 
  {
    150.000000,	300.000000,	0.000000,
    300.000000,	300.000000,	0.000000,
    300.000000,	150.000000,	0.000000,
    150.000000,	150.000000,	0.000000,
    150.000000,	300.000000,	-150.000000,
    300.000000,	300.000000,	-150.000000,
    300.000000,	150.000000,	-150.000000,
    150.000000,	150.000000,	-150.000000,
  };

  const int aiTriNode [] = 
  {
    4,3,1,
    3,2,1,
    7,8,5,
    6,7,5,
    6,1,2,
    5,1,6,
    7,2,3,
    6,2,7,
    8,7,4,
    7,3,4,
    5,8,1,
    8,4,1,
  };


  //Initialize meshIn (Input Structure for Meshing)
  t3dMeshIo meshIn (iNoPts, iNoTri); 

  //Initialize meshOut (Output Structure used to store the resulting Mesh)
  t3dMeshIo meshOut (iNoPts, iNoTri);   
    
  //Copy surface mesh data into meshIn 
  iRet = meshIn.copySurfaceMesh (iNoPts, adCoord, iNoTri, aiTriNode);
  if (iRet !=0)
    goto Error;
    
  //Set various meshing Options
  meshIn.bAspect = false;      
  meshIn.dAspect = 5.0;

  meshIn.bVolume  = false;
  meshIn.iNoVolumeElem = 100;
    
  meshIn.bGrid = false;  
  meshIn.iNoXPts = 5;
  meshIn.iNoYPts = 5;
  meshIn.iNoZPts = 5;
  meshIn.bDoConvexHull = false;
  meshIn.bWriteOpFile = true;

    
  //Main call to create 3D tetrahedral Mesh
  eResult = eFailed;
  iRet = t3dCreateTetMesh (&meshIn, &meshOut, &eResult);
  if (iRet !=0)
    goto Error;


  if (eResult == eSuccess)  //Success in Meshing
  {
    //Code to demo copying of output data from Meshing
    double *padCoordOut = NULL;
    int* paiTetNode = NULL;

    padCoordOut = new double[meshOut.iNoPointOut*3*sizeof(double)];
    paiTetNode = new int [meshOut.iNoElemOut*4*sizeof(int)];

    //Copy output of Meshing to padCoordOut and paiTetNode
    memcpy (padCoordOut, meshOut.adPointOut, meshOut.iNoPointOut*3*sizeof(double));
    memcpy (paiTetNode, meshOut.aiElemOut, meshOut.iNoElemOut*4*sizeof(int));

    //At this point, the tetrahedra information is copies into padCoordOut and paiTetNode.
    delete [] padCoordOut;
    delete[] paiTetNode;
  }
  else
  {
    //Error in Mesh Generation
  }

goto Exit;
Error:
iRet = -1;
goto Exit;
Exit:
return iRet;
}
